﻿using System;
using System.Web.Http;
using System.Web.Http.Dispatcher;
using System.Collections.Generic;
using System.Threading.Tasks;
using Owin;
using Microsoft.Owin;
using Microsoft.Owin.Cors;
using C1.Web.Api.Report;
using C1.Web.Api.Storage;
using System.IO;

[assembly: OwinStartup(typeof(ReportServie_FLXR.Startup))]

namespace ReportServie_FLXR
{
    /// <summary>
    /// Owin startup class.
    /// </summary>
    public class Startup
    {
        private readonly HttpConfiguration config = GlobalConfiguration.Configuration;

    public void Configuration(IAppBuilder app)
        {
            // Tracing support
            //config.EnableSystemDiagnosticsTracing();

            // CORS supports - does not apply to <form>
            app.UseCors(CorsOptions.AllowAll);
            app.UseReportProviders()
                .AddFlexReportDiskStorage("ReportsRoot", GetFullRoot("Files"));

            // Web Api
            RegisterRoutes(config);
            app.UseWebApi(config);

            // Anything unhandled
            app.Run(context =>
            {
                context.Response.Redirect(context.Request.PathBase + "/default.html");
                return Task.FromResult(0);
            });
        }


        private static string GetFullRoot(string root)
        {
            var applicationBase = AppDomain.CurrentDomain.SetupInformation.ApplicationBase;
            var fullRoot = Path.GetFullPath(Path.Combine(applicationBase, root));
            if (!fullRoot.EndsWith(Path.DirectorySeparatorChar.ToString(), StringComparison.Ordinal))
            {
                fullRoot += Path.DirectorySeparatorChar;
            }
            return fullRoot;
        }


        private static void RegisterRoutes(HttpConfiguration config)
        {
            // Use APIs defined in the C1 library
            config.Services.Replace(typeof(IAssembliesResolver), new CustomAssembliesResolver());

            // Attribute Routes
            config.MapHttpAttributeRoutes();

            // default route
            config.Routes.MapHttpRoute(
                name: "DefaultApi",
                routeTemplate: "api/{controller}/{id}",
                defaults: new { id = RouteParameter.Optional }
            );
        }
    }

    internal class CustomAssembliesResolver : DefaultAssembliesResolver
    {
        public override ICollection<System.Reflection.Assembly> GetAssemblies()
        {
            var assemblies = base.GetAssemblies();
            var customControllersAssembly = typeof(StorageController).Assembly;
            if (!assemblies.Contains(customControllersAssembly))
            {
                assemblies.Add(customControllersAssembly);
            }

            customControllersAssembly = typeof(ReportController).Assembly;
            if (!assemblies.Contains(customControllersAssembly))
            {
                assemblies.Add(customControllersAssembly);
            }

            return assemblies;
        }
    }
}